SystemOrganization addCategory: #Everett!
SystemOrganization addCategory: #'Everett-Tests'!

TestCase subclass: #EntityManagerTestCase
	instanceVariableNames: 'em'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

EntityManagerTestCase class
	instanceVariableNames: 'em'!

!EntityManagerTestCase methodsFor: 'running' stamp: 'EricShahkarami 7/2/2013 11:47'!
setUp 

	em := EntityManager new.! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'EricShahkarami 7/4/2013 11:12'!
testEntitiesAtPutComponent

	| e1 e2  pc1  bc1 c1 c2 s1  hc1 plc1 s2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	hc1 := HealthComponent health: 5.
	bc1 := VelocityComponent velocity: 1@1.
	plc1 := PlayerComponent player: 1.
		
	em addComponent: plc1 toEntity: e1.
	em 	entityAt: e1 putComponent: bc1.
		
	em addComponent: hc1 toEntity: e2.	
	em 	entityAt: e2 putComponent:  bc1.
	em 	entityAt: e2 putComponent:  pc1.
		
	c1 := Set newFrom: #(#player #velocity).
	c2:= Set newFrom:#(#health #velocity #position).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesMatching: c1) = s1 ).
	self assert: ( (em entitiesMatching: c2) = s2 ).! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'EricShahkarami 7/4/2013 11:11'!
testEntityMatches

	| e1 e2 pc1 bc1 bc2 c1 c2 c3 s1 s2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	bc1 := VelocityComponent velocity: 1@1.
	bc2 := VelocityComponent velocity: 2@0.
	
	em addComponent: pc1 toEntity: e1.
	em addComponent: bc1 toEntity: e1.
	em addComponent: bc2 toEntity: e2.
	
	c1 := Set newFrom: #(#position).
	c2 := Set newFrom: #(#velocity).
	c3 := Set newFrom: #(#position #velocity).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesMatching: c1) = s1 ).
	self assert: ( (em entitiesMatching: c2) = s2 ).
	self assert: ( (em entitiesMatching: c3) = s1 )! !

!EntityManagerTestCase methodsFor: 'testing' stamp: 'EricShahkarami 7/4/2013 11:11'!
testRemoveEntityComponent

	| e1 e2 pc1  bc1 c1 c2 s1 s2  hc1 plc1 plc2 |
	
	e1 := em createEntity.
	e2 := em createEntity.

	pc1 := PositionComponent position: 3@7.
	hc1 := HealthComponent health: 5.
	bc1 := VelocityComponent velocity: 1@1.
	plc1 := PlayerComponent player: 1.
	plc2 := PlayerComponent player: 2.
	
	em addComponent: pc1 toEntity: e1.
	em addComponent: hc1 toEntity: e1.
	em removeComponent: (PositionComponent identity) fromEntity: e1.
	
	em addComponent: plc2 toEntity: e2.
	em addComponent: bc1 toEntity: e2.
	em removeComponent: (PlayerComponent identity) fromEntity: e2.
	em addComponent: plc1 toEntity: e2.
	
	c1 := Set newFrom: #(#player #velocity).
	c2:= Set newFrom: #(#health).

	s1 := Set new.
	s2 := Set new.
	s1 add: e1.
	s2 add: e2.
	
	self assert: ( (em entitiesMatching: c2) = s1 ).
	self assert: ( (em entitiesMatching: c1) = s2 ).

	
	
	! !

TestCase subclass: #SystemManagerTestCase
	instanceVariableNames: 'em sm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:51'!
setUp

	em := EntityManager new.
	sm := SystemManager new! !

!SystemManagerTestCase methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/5/2013 15:55'!
testSystemUpdate

	"Tests that e1 and e2 have their positions updated but not e3, which doesn't have a velocity component"

	| e1 e2 e3 pc1 pc2 pc3 vc1 vc2 |
	
	e1 := em createEntity.
	e2 := em createEntity.
	e3 := em createEntity.

	pc1 := PositionComponent position: 3@4.
	pc2 := PositionComponent position: 1@1.
	pc3 := PositionComponent position: 9@9.
	vc1 := VelocityComponent velocity: 1@1.
	vc2 := VelocityComponent velocity: 2@0.

	em addComponent: pc1 toEntity: e1.
	em addComponent: pc2 toEntity: e2.
	em addComponent: pc3 toEntity: e3.
	em addComponent: vc1 toEntity: e1.
	em addComponent: vc2 toEntity: e2.
	
	sm addSystem: (MovementSystem new).
	sm updateEntitiesOf: em afterDeltaTime: 1.
	
	self assert: ( (pc1 position) = (4@5) ).
	self assert: ( (pc2 position) = (3@1) ).
	self assert: ( (pc3 position) = (9@9) )! !

Object subclass: #EntityManager
	instanceVariableNames: 'nextAvailableEntity entityTable entityTags changedEntities removedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/2/2013 09:38'!
addComponent: aComponentInstance toEntity: anEntity

	(anEntity < nextAvailableEntity and: anEntity >= 0) ifTrue: 
	[  
		self entityAt: anEntity putComponent: aComponentInstance.
		changedEntities add: anEntity
	]! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/1/2013 10:49'!
changedEntities
	^changedEntities ! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 09:38'!
component: aComponentIdentifier fromEntity: anEntity

	| componentsOfEntity |
	
	componentsOfEntity := self entityAt: anEntity.
	^(componentsOfEntity at: aComponentIdentifier)! !

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/1/2013 10:35'!
createEntity

	entityTable at: nextAvailableEntity put: (Dictionary new).	
	changedEntities add: nextAvailableEntity.
	nextAvailableEntity := nextAvailableEntity + 1.
	^(nextAvailableEntity - 1)
! !

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/2/2013 08:54'!
createEntityFromTag: aTag

	"Create a new entity and add the tagged entity's components to it"
	| entity components |
	
	entity := self createEntity.
	components := (entityTags at: aTag) deepCopy.
	entityTable at: entity put: components.
	^entity
	
	! !

!EntityManager methodsFor: 'testing' stamp: 'EricShahkarami 7/5/2013 08:34'!
doesEntity: anEntity containComponents: aSetOfComponents

	| entityComponents |
	
	entityComponents := (self entityAt: anEntity) keys asSet.
	^( (aSetOfComponents intersection: entityComponents) = aSetOfComponents )! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:23'!
entitiesMatching: aSetOfComponents

	^(self entitiesSelect: [ :entity | self doesEntity: entity containComponents: aSetOfComponents ])! !

!EntityManager methodsFor: 'enumerate' stamp: 'EricShahkarami 7/2/2013 11:22'!
entitiesSelect: aBlock

	| selectEntities |
	
	selectEntities := Set new.
	entityTable keysDo: [ :entity | (aBlock value: entity) ifTrue: [ selectEntities add: entity ]  ].
	^selectEntities! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity
	
	^(entityTable at: anEntity)! !

!EntityManager methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 09:38'!
entityAt: anEntity putComponent: aComponentInstance 
		
	(entityTable at: anEntity) at: (aComponentInstance class identity) put: aComponentInstance. ! !

!EntityManager methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/2/2013 16:08'!
initialize 
	
	super initialize.
	entityTable := Dictionary new.
	entityTags := Dictionary new.
	changedEntities := Bag new.
	removedEntities := Set new.
	nextAvailableEntity := 0! !

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/2/2013 09:38'!
removeComponent: aComponentIdentifier fromEntity: anEntity

	| components |
	
	components := self entityAt: anEntity.
	components removeKey: aComponentIdentifier.
	changedEntities add: anEntity
! !

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/1/2013 11:17'!
removeEntity: anEntity	
	"Removes entity from entityTable and places entity in removed pile."	
	
	entityTable removeKey: anEntity.
	removedEntities add: anEntity
	! !

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/1/2013 22:24'!
removeTag: aTag

	entityTags removeKey: aTag.! !

!EntityManager methodsFor: 'accessing' stamp: 'EricShahkarami 7/1/2013 10:49'!
removedEntities
	^removedEntities ! !

!EntityManager methodsFor: 'modify' stamp: 'EricShahkarami 7/2/2013 08:52'!
tagEntity: anEntity with: aTag

	"aTag is a symbol. Creates a copy of entity's components."
	| componentCopies |
	
	componentCopies := Dictionary new.
	componentCopies := (entityTable at: anEntity) deepCopy.
	entityTags at: aTag put: componentCopies.


! !

Object subclass: #EverettComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 13:52'!
identity
	^self subclassResponsibility! !

EverettComponent subclass: #HealthComponent
	instanceVariableNames: 'health'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!HealthComponent class methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:53'!
health: healthLevel

	^(self new health: healthLevel)! !

!HealthComponent class methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:54'!
identity

	^#health! !

!HealthComponent methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:34'!
health
	^health! !

!HealthComponent methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:33'!
health: healthlevel 
	
	health := healthlevel! !

EverettComponent subclass: #PlayerComponent
	instanceVariableNames: 'player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!PlayerComponent class methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:50'!
identity
	^#player! !

!PlayerComponent class methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:52'!
player: aPlayer

	^(self new player: aPlayer)! !

!PlayerComponent methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:35'!
player
	
	^player ! !

!PlayerComponent methodsFor: 'as yet unclassified' stamp: 'AnkuAdhikari 7/2/2013 15:35'!
player: aPlayer 
	
	player := aPlayer! !

EverettComponent subclass: #PositionComponent
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!PositionComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:09'!
identity
	^#position! !

!PositionComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
position: aPosition

	^(self new position: aPosition)! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
position

	^position! !

!PositionComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 10:02'!
position: aPosition 
	
	position := aPosition! !

EverettComponent subclass: #VelocityComponent
	instanceVariableNames: 'velocity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!VelocityComponent class methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:27'!
identity 	
	^#velocity! !

!VelocityComponent class methodsFor: 'instance creation' stamp: 'EricShahkarami 7/2/2013 11:43'!
velocity: aVelocity

	^(self new velocity: aVelocity)! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity

	^velocity ! !

!VelocityComponent methodsFor: 'accessing' stamp: 'EricShahkarami 7/2/2013 11:28'!
velocity: aVelocity
	
	velocity := aVelocity! !

Object subclass: #EverettSystem
	instanceVariableNames: 'updateInterval interestedComponents interestedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!EverettSystem class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 14:20'!
identity
	^self subclassResponsibility ! !

!EverettSystem class methodsFor: 'instance creation' stamp: 'EricShahkarami 6/27/2013 16:26'!
updateInterval: anUpdateInterval
	^(self new updateInterval: anUpdateInterval)! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/2/2013 16:07'!
initialize

	updateInterval = 0.
	interestedComponents := Set new.
	interestedEntities := Set new! !

!EverettSystem methodsFor: 'accessing' stamp: 'EricShahkarami 7/5/2013 11:28'!
interestedEntities

	^interestedEntities! !

!EverettSystem methodsFor: 'private' stamp: 'EricShahkarami 7/5/2013 11:44'!
resolveChangedEntitiesFrom: anEntityManager

	| changedEntities |
	
	changedEntities := anEntityManager changedEntities.
	(changedEntities asSet) do: [ :entity | self verifyEntity: entity from: anEntityManager ].! !

!EverettSystem methodsFor: 'private' stamp: 'EricShahkarami 7/2/2013 15:17'!
resolveRemovedEntitiesFrom: anEntityManager 

	| removedEntities |
	
	removedEntities := anEntityManager removedEntities.
	interestedEntities do: [ :entity | (removedEntities include: entity) ifTrue: [ interestedEntities remove: entity ] ]! !

!EverettSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 11:38'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt
								
	self resolveRemovedEntitiesFrom: anEntityManager.
	self resolveChangedEntitiesFrom: anEntityManager.
		
	interestedEntities do: [ :entity | self updateEntity: entity from: anEntityManager afterDeltaTime: dt ]! !

!EverettSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 11:12'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	^self subclassResponsibility! !

!EverettSystem methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/2/2013 14:46'!
updateInterval: anUpdateInterval
	
	updateInterval := anUpdateInterval ! !

!EverettSystem methodsFor: 'testing' stamp: 'EricShahkarami 7/5/2013 11:32'!
verifyEntity: anEntity from: anEntityManager

	(anEntityManager doesEntity: anEntity containComponents: interestedComponents) 
	ifTrue: 
	[  
		(interestedEntities includes: anEntity) ifFalse: [ interestedEntities add: anEntity ].
	]
	ifFalse:
	[ 
		(interestedEntities includes: anEntity) ifTrue: [ interestedEntities remove: anEntity ]
	]! !

EverettSystem subclass: #MovementSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett-Tests'!

!MovementSystem commentStamp: '<historical>' prior: 0!
I am a system that updates the position of entities.!

!MovementSystem class methodsFor: 'as yet unclassified' stamp: 'EricShahkarami 7/4/2013 10:55'!
identity 

	^#movement! !

!MovementSystem methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/5/2013 11:19'!
initialize
	
	super initialize.
	interestedComponents add: (PositionComponent identity); add: (VelocityComponent identity).! !

!MovementSystem methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 15:56'!
updateEntity: anEntity from: anEntityManager afterDeltaTime: dt

	"Adds velocity vector of entity to position of entity, scaled by dt."
	| pc vc |
	
	pc := anEntityManager component: (PositionComponent identity) fromEntity: anEntity.
	vc := anEntityManager component: (VelocityComponent identity) fromEntity: anEntity.	
		
	pc position: ( pc position + (dt * (vc velocity)) )! !

Object subclass: #SystemManager
	instanceVariableNames: 'systems priorities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Everett'!

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/1/2013 15:18'!
addSystem: aSystemInstance 
	
	self addSystem: aSystemInstance withPriority: 0 ! !

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/2/2013 11:29'!
addSystem: aSystemInstance withPriority: aPriority

	systems at: (aSystemInstance class identity) put: aSystemInstance.
	priorities add: ( (aSystemInstance class identity)->aPriority )! !

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/1/2013 15:53'!
assignSystem: aSystemIdentifier aPriorityOf: aPriority

	priorities add: (aSystemIdentifier->aPriority)! !

!SystemManager methodsFor: 'initialize-release' stamp: 'EricShahkarami 7/1/2013 15:51'!
initialize
	
	super initialize.
	systems := Dictionary new.
	priorities := SortedCollection sortBlock: [ :p1 :p2 | p1 value <= p2 value ]! !

!SystemManager methodsFor: 'create-modify-destroy' stamp: 'EricShahkarami 7/2/2013 11:30'!
removeSystem: aSystemIdentifier

	| systemPriority |

	systems removeKey: aSystemIdentifier.
	systemPriority := priorities select: [ :assoc | assoc key = aSystemIdentifier ].
	priorities remove: (systemPriority at: 1)! !

!SystemManager methodsFor: 'processing' stamp: 'EricShahkarami 7/5/2013 11:21'!
updateEntitiesOf: anEntityManager afterDeltaTime: dt

	| changedEntities removedEntities |

	"priorities do: [ :assoc | (systems at: (assoc key)) updateEntitiesOf: anEntityManager afterDeltaTime: dt ]."
	systems do: [ :system | system updateEntitiesOf: anEntityManager afterDeltaTime: dt ].
		
	"Add setters for changed/removed entities to EntityManager"
	changedEntities := anEntityManager changedEntities.
	removedEntities := anEntityManager removedEntities.
	(changedEntities asSet) do: [ :entity | changedEntities remove: entity ].
	removedEntities do: [ :entity | removedEntities remove: entity ]
	
	! !
